(function() {
    if (window.NativeBridge) {
        return;
    }

    var _jsCallbacks = {};
    var _sendMessageQueue = [];
    var _messageHandlers = {};
    var _uniqueId = 1;

    function _call(handlerName, data, responseCallback) {
        _doCall(handlerName, data, responseCallback);
    }

    function _doCall(handlerName, data, responseCallback) {
        if(!data){
          data = {};
        }

        var callbackId;
        if (typeof responseCallback == 'function') {
            var callbackId = 'cb_' + (_uniqueId++) + '_' + new Date().getTime();
            _jsCallbacks[callbackId] = responseCallback;
        }

        var msgObj = {'moduleName':handlerName, 'callbackId': callbackId, 'data':data};
        _sendMessage(msgObj);
    }

    function _sendMessage(message){
        _sendMessageQueue.push(message);
        prompt(JSON.stringify(message), "$sendToNative:");
    }

    function _handleMessageFromNative(messageJson){
        setTimeout(function(){
          var message = JSON.parse(messageJson);
          var jsCallback;
          //call js 回调
          if(message.callbackId){
             jsCallback = _jsCallbacks[message.callbackId];

             if(!jsCallback){
               return;
             }

             jsCallback(message.data);
             delete _jsCallbacks[message.callbackId];
          }
           //call js 方法
           else if(message.moduleName){
                var handler = _messageHandlers[message.moduleName];

                if(!handler){
                   console.log("handler not found : " + message.moduleName);
                }
                try{
                    handler(message.data);
                } catch (exception){
                  console.log("Web call handler failed : " + JSON.stringify(message));
                }
          }
        })
    }

    function _registerHandler(handlerName, handler){
      _messageHandlers[handlerName] = handler;
    }


    function _attachFunc(name,callback){
      NativeBridge[name] = function(data){
          _call(name, data, callback);
      }
    }

    function _rename(newName){
       if(NativeBridge && !window[newName]){
         window[newName] = NativeBridge;
         NativeBridge = undefinded;
         return true
       }
       return false;
    }

    function _log(tag, message){
      var message = {"moduleName":tag, 'data':{'tag':tag,'message':message}};
       window.Native.invoke(JSON.stringify(message));
    }

    var NativeBridge = window.NativeBridge = {
        invoke: _call,
        log: _log,
        registerHandler: _registerHandler,
        handleMessageFromNative:_handleMessageFromNative,
        rename: _rename,
        attachFunc: _attachFunc
    };

    var doc = document;
    var readyEvent = doc.createEvent('Events');
    readyEvent.initEvent('NativeBridgeReady');
    readyEvent.bridge = NativeBridge;
    doc.dispatchEvent(readyEvent);

    console.log("NativeBridge ready");
})();